/**
 *  \file RMF/physics_decorators.h
 *  \brief Helper functions for manipulating RMF files.
 *
 *  Copyright 2007-2013 IMP Inventors. All rights reserved.
 *
 */

#ifndef RMF_PHYSICS_DECORATORS_H
#define RMF_PHYSICS_DECORATORS_H

#include <RMF/config.h>
#include <RMF/infrastructure_macros.h>
#include <RMF/NodeHandle.h>
#include <RMF/FileHandle.h>
#include <RMF/Decorator.h>
#include <RMF/Factory.h>
#include <RMF/constants.h>
#include <RMF/internal/utility.h>
#include <RMF/internal/paths.h>
#include <boost/array.hpp>
#include <boost/lexical_cast.hpp>
#include "alias_decorators.h" 

RMF_ENABLE_WARNINGS
namespace RMF {


  /** These particles has associated coordinates and radius information.

       See also Particle and ParticleConstFactory.
    */
  class ParticleConst: public Decorator<NodeConstHandle> {
    friend class ParticleConstFactory;
    std::string get_name() const {return "Particle";}
  private:
    typedef Decorator<NodeConstHandle> P;
FloatKeys coordinates_;
FloatKey mass_;
FloatKey radius_;
    ParticleConst(NodeConstHandle nh,
              FloatKeys coordinates,
FloatKey mass,
FloatKey radius):
       P(nh),
coordinates_(coordinates),
mass_(mass),
radius_(radius) {
    }
  public:
  /** The coordinates in angstroms. */
  Floats get_coordinates() const {
    try {
      Floats ret(coordinates_.size());
      for (unsigned int i = 0; i < coordinates_.size(); ++i) {
        ret[i] = P::get_value(coordinates_[i]);
      }
      return ret;
    } RMF_DECORATOR_CATCH( );
  }

  /** The mass in Daltons. */
  Float get_mass() const {
    try {
      return P::get_value(mass_);
    } RMF_DECORATOR_CATCH( );
  }

  /** The radius in angstroms. */
  Float get_radius() const {
    try {
      return P::get_value(radius_);
    } RMF_DECORATOR_CATCH( );
  }

    static std::string get_decorator_type_name() {
         return "ParticleConst";
    }
  };

    typedef std::vector<ParticleConst> ParticleConsts;

  /** These particles has associated coordinates and radius information.

       See also ParticleConst and ParticleFactory.
    */
  class Particle: public Decorator<NodeHandle> {
    friend class ParticleFactory;
    std::string get_name() const {return "Particle";}
  private:
    typedef Decorator<NodeHandle> P;
FloatKeys coordinates_;
FloatKey mass_;
FloatKey radius_;
    Particle(NodeHandle nh,
              FloatKeys coordinates,
FloatKey mass,
FloatKey radius):
       P(nh),
coordinates_(coordinates),
mass_(mass),
radius_(radius) {
    }
  public:
  /** The coordinates in angstroms. */
  Floats get_coordinates() const {
    try {
      Floats ret(coordinates_.size());
      for (unsigned int i = 0; i < coordinates_.size(); ++i) {
        ret[i] = P::get_value(coordinates_[i]);
      }
      return ret;
    } RMF_DECORATOR_CATCH( );
  }
  /** The coordinates in angstroms. */
  void set_coordinates(Floats v) {
    try {
      for (unsigned int i = 0; i< coordinates_.size(); ++i) {
         P::set_value(coordinates_[i], v[i]);
      }
    } RMF_DECORATOR_CATCH( );
  }

  /** The mass in Daltons. */
  Float get_mass() const {
    try {
      return P::get_value(mass_);
    } RMF_DECORATOR_CATCH( );
  }
  /** The mass in Daltons. */
  void set_mass(Float v) {
    try {
      P::set_value(mass_, v);
    } RMF_DECORATOR_CATCH( );
  }


  /** The radius in angstroms. */
  Float get_radius() const {
    try {
      return P::get_value(radius_);
    } RMF_DECORATOR_CATCH( );
  }
  /** The radius in angstroms. */
  void set_radius(Float v) {
    try {
      P::set_value(radius_, v);
    } RMF_DECORATOR_CATCH( );
  }


    static std::string get_decorator_type_name() {
         return "Particle";
    }
  };

    typedef std::vector<Particle> Particles;

  /** Create decorators of type Particle.

       See also ParticleConst and ParticleFactory.
    */
  class ParticleConstFactory: public Factory<FileConstHandle> {
    typedef Factory<FileConstHandle> P;
    Category cat_;
FloatKeys coordinates_;
FloatKey mass_;
FloatKey radius_;

  FloatKeys get_coordinates_keys(FileConstHandle fh) {
    FloatKeys ret;
    ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian x"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian y"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian z"));;
    return ret;
  }
  bool get_has_coordinates_attributes(NodeConstHandle nh) const {
    for (unsigned int i = 0; i< coordinates_.size(); ++i) {
      if (!nh.get_has_value(coordinates_[i])) return false;
    }
    return true;
  }



  public:
    ParticleConstFactory(FileConstHandle fh):
    P(),
    cat_(fh.get_category("physics")),
    coordinates_(get_coordinates_keys(fh)), mass_(fh.get_key<FloatTraits>(cat_, "mass")), radius_(fh.get_key<FloatTraits>(cat_, "radius")) {
    }
    /** Get a ParticleConst for nh.*/
    ParticleConst get(NodeConstHandle nh) const {
      RMF_USAGE_CHECK((nh.get_type() == RMF::REPRESENTATION), std::string("Bad node type. Got \"")
                                      + boost::lexical_cast<std::string>(nh.get_type())
                                      + "\" in decorator type  Particle");
      return ParticleConst(nh, coordinates_,
mass_,
radius_);
    }
    /** Check whether nh has all the attributes required to be a
        ParticleConst.*/
    bool get_is(NodeConstHandle nh) const {
      return (nh.get_type() == RMF::REPRESENTATION)
    && nh.get_has_value(coordinates_[0]) && nh.get_has_value(coordinates_[1]) && nh.get_has_value(coordinates_[2])
    && nh.get_has_value(mass_)
    && nh.get_has_value(radius_);
    }
    
  /** Get values from a list of NodeIDs all at the same time. This method
      is a bit experimental and may change.
    */
  FloatsList get_coordinates(FileConstHandle fh,
                       const NodeIDs &node_ids) const {
      FloatsList ret(node_ids.size());
      for (unsigned int i = 0; i< node_ids.size(); ++i) {
        NodeConstHandle nh = fh.get_node(node_ids[i]);
        ret[i].resize(coordinates_.size());
        for (unsigned int j = 0; j < coordinates_.size(); ++j) {
          ret[i][j] = nh.get_value(coordinates_[j]);
        }
      }
      return ret;
  }



  };

  typedef std::vector<ParticleConstFactory> ParticleConstFactories;

  /** Create decorators of type Particle.

       See also Particle and ParticleConstFactory.
    */
  class ParticleFactory: public Factory<FileHandle> {
    typedef Factory<FileHandle> P;
    Category cat_;
FloatKeys coordinates_;
FloatKey mass_;
FloatKey radius_;

  FloatKeys get_coordinates_keys(FileHandle fh) {
    FloatKeys ret;
    ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian x"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian y"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian z"));;
    return ret;
  }
  bool get_has_coordinates_attributes(NodeConstHandle nh) const {
    for (unsigned int i = 0; i< coordinates_.size(); ++i) {
      if (!nh.get_has_value(coordinates_[i])) return false;
    }
    return true;
  }



  public:
    ParticleFactory(FileHandle fh):
    P(),
    cat_(fh.get_category("physics")),
    coordinates_(get_coordinates_keys(fh)), mass_(fh.get_key<FloatTraits>(cat_, "mass")), radius_(fh.get_key<FloatTraits>(cat_, "radius")) {
    }
    /** Get a Particle for nh.*/
    Particle get(NodeHandle nh) const {
      RMF_USAGE_CHECK((nh.get_type() == RMF::REPRESENTATION), std::string("Bad node type. Got \"")
                                      + boost::lexical_cast<std::string>(nh.get_type())
                                      + "\" in decorator type  Particle");
                                      
                                      
      return Particle(nh, coordinates_,
mass_,
radius_);
    }
    /** Check whether nh has all the attributes required to be a
        Particle.*/
    bool get_is(NodeHandle nh) const {
      return (nh.get_type() == RMF::REPRESENTATION)
    && nh.get_has_value(coordinates_[0]) && nh.get_has_value(coordinates_[1]) && nh.get_has_value(coordinates_[2])
    && nh.get_has_value(mass_)
    && nh.get_has_value(radius_);
    }
    
  /** Get values from a list of NodeIDs all at the same time. This method
      is a bit experimental and may change.
    */
  FloatsList get_coordinates(FileConstHandle fh,
                       const NodeIDs &node_ids) const {
      FloatsList ret(node_ids.size());
      for (unsigned int i = 0; i< node_ids.size(); ++i) {
        NodeConstHandle nh = fh.get_node(node_ids[i]);
        ret[i].resize(coordinates_.size());
        for (unsigned int j = 0; j < coordinates_.size(); ++j) {
          ret[i][j] = nh.get_value(coordinates_[j]);
        }
      }
      return ret;
  }



  };

  typedef std::vector<ParticleFactory> ParticleFactories;


  /** These particles have associated coordinates and radius information.

       See also IntermediateParticle and IntermediateParticleConstFactory.
    */
  class IntermediateParticleConst: public Decorator<NodeConstHandle> {
    friend class IntermediateParticleConstFactory;
    std::string get_name() const {return "IntermediateParticle";}
  private:
    typedef Decorator<NodeConstHandle> P;
FloatKeys coordinates_;
FloatKey radius_;
    IntermediateParticleConst(NodeConstHandle nh,
              FloatKeys coordinates,
FloatKey radius):
       P(nh),
coordinates_(coordinates),
radius_(radius) {
    }
  public:
  /** The coordinates in angstroms. */
  Floats get_coordinates() const {
    try {
      Floats ret(coordinates_.size());
      for (unsigned int i = 0; i < coordinates_.size(); ++i) {
        ret[i] = P::get_value(coordinates_[i]);
      }
      return ret;
    } RMF_DECORATOR_CATCH( );
  }

  /** The radius in angstroms. */
  Float get_radius() const {
    try {
      return P::get_value(radius_);
    } RMF_DECORATOR_CATCH( );
  }

    static std::string get_decorator_type_name() {
         return "IntermediateParticleConst";
    }
  };

    typedef std::vector<IntermediateParticleConst> IntermediateParticleConsts;

  /** These particles have associated coordinates and radius information.

       See also IntermediateParticleConst and IntermediateParticleFactory.
    */
  class IntermediateParticle: public Decorator<NodeHandle> {
    friend class IntermediateParticleFactory;
    std::string get_name() const {return "IntermediateParticle";}
  private:
    typedef Decorator<NodeHandle> P;
FloatKeys coordinates_;
FloatKey radius_;
    IntermediateParticle(NodeHandle nh,
              FloatKeys coordinates,
FloatKey radius):
       P(nh),
coordinates_(coordinates),
radius_(radius) {
    }
  public:
  /** The coordinates in angstroms. */
  Floats get_coordinates() const {
    try {
      Floats ret(coordinates_.size());
      for (unsigned int i = 0; i < coordinates_.size(); ++i) {
        ret[i] = P::get_value(coordinates_[i]);
      }
      return ret;
    } RMF_DECORATOR_CATCH( );
  }
  /** The coordinates in angstroms. */
  void set_coordinates(Floats v) {
    try {
      for (unsigned int i = 0; i< coordinates_.size(); ++i) {
         P::set_value(coordinates_[i], v[i]);
      }
    } RMF_DECORATOR_CATCH( );
  }

  /** The radius in angstroms. */
  Float get_radius() const {
    try {
      return P::get_value(radius_);
    } RMF_DECORATOR_CATCH( );
  }
  /** The radius in angstroms. */
  void set_radius(Float v) {
    try {
      P::set_value(radius_, v);
    } RMF_DECORATOR_CATCH( );
  }


    static std::string get_decorator_type_name() {
         return "IntermediateParticle";
    }
  };

    typedef std::vector<IntermediateParticle> IntermediateParticles;

  /** Create decorators of type IntermediateParticle.

       See also IntermediateParticleConst and IntermediateParticleFactory.
    */
  class IntermediateParticleConstFactory: public Factory<FileConstHandle> {
    typedef Factory<FileConstHandle> P;
    Category cat_;
FloatKeys coordinates_;
FloatKey radius_;

  FloatKeys get_coordinates_keys(FileConstHandle fh) {
    FloatKeys ret;
    ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian x"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian y"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian z"));;
    return ret;
  }
  bool get_has_coordinates_attributes(NodeConstHandle nh) const {
    for (unsigned int i = 0; i< coordinates_.size(); ++i) {
      if (!nh.get_has_value(coordinates_[i])) return false;
    }
    return true;
  }


  public:
    IntermediateParticleConstFactory(FileConstHandle fh):
    P(),
    cat_(fh.get_category("physics")),
    coordinates_(get_coordinates_keys(fh)), radius_(fh.get_key<FloatTraits>(cat_, "radius")) {
    }
    /** Get a IntermediateParticleConst for nh.*/
    IntermediateParticleConst get(NodeConstHandle nh) const {
      RMF_USAGE_CHECK((nh.get_type() == RMF::REPRESENTATION), std::string("Bad node type. Got \"")
                                      + boost::lexical_cast<std::string>(nh.get_type())
                                      + "\" in decorator type  IntermediateParticle");
      return IntermediateParticleConst(nh, coordinates_,
radius_);
    }
    /** Check whether nh has all the attributes required to be a
        IntermediateParticleConst.*/
    bool get_is(NodeConstHandle nh) const {
      return (nh.get_type() == RMF::REPRESENTATION)
    && nh.get_has_value(coordinates_[0]) && nh.get_has_value(coordinates_[1]) && nh.get_has_value(coordinates_[2])
    && nh.get_has_value(radius_);
    }
    
  /** Get values from a list of NodeIDs all at the same time. This method
      is a bit experimental and may change.
    */
  FloatsList get_coordinates(FileConstHandle fh,
                       const NodeIDs &node_ids) const {
      FloatsList ret(node_ids.size());
      for (unsigned int i = 0; i< node_ids.size(); ++i) {
        NodeConstHandle nh = fh.get_node(node_ids[i]);
        ret[i].resize(coordinates_.size());
        for (unsigned int j = 0; j < coordinates_.size(); ++j) {
          ret[i][j] = nh.get_value(coordinates_[j]);
        }
      }
      return ret;
  }


  };

  typedef std::vector<IntermediateParticleConstFactory> IntermediateParticleConstFactories;

  /** Create decorators of type IntermediateParticle.

       See also IntermediateParticle and IntermediateParticleConstFactory.
    */
  class IntermediateParticleFactory: public Factory<FileHandle> {
    typedef Factory<FileHandle> P;
    Category cat_;
FloatKeys coordinates_;
FloatKey radius_;

  FloatKeys get_coordinates_keys(FileHandle fh) {
    FloatKeys ret;
    ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian x"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian y"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian z"));;
    return ret;
  }
  bool get_has_coordinates_attributes(NodeConstHandle nh) const {
    for (unsigned int i = 0; i< coordinates_.size(); ++i) {
      if (!nh.get_has_value(coordinates_[i])) return false;
    }
    return true;
  }


  public:
    IntermediateParticleFactory(FileHandle fh):
    P(),
    cat_(fh.get_category("physics")),
    coordinates_(get_coordinates_keys(fh)), radius_(fh.get_key<FloatTraits>(cat_, "radius")) {
    }
    /** Get a IntermediateParticle for nh.*/
    IntermediateParticle get(NodeHandle nh) const {
      RMF_USAGE_CHECK((nh.get_type() == RMF::REPRESENTATION), std::string("Bad node type. Got \"")
                                      + boost::lexical_cast<std::string>(nh.get_type())
                                      + "\" in decorator type  IntermediateParticle");
                                      
                                      
      return IntermediateParticle(nh, coordinates_,
radius_);
    }
    /** Check whether nh has all the attributes required to be a
        IntermediateParticle.*/
    bool get_is(NodeHandle nh) const {
      return (nh.get_type() == RMF::REPRESENTATION)
    && nh.get_has_value(coordinates_[0]) && nh.get_has_value(coordinates_[1]) && nh.get_has_value(coordinates_[2])
    && nh.get_has_value(radius_);
    }
    
  /** Get values from a list of NodeIDs all at the same time. This method
      is a bit experimental and may change.
    */
  FloatsList get_coordinates(FileConstHandle fh,
                       const NodeIDs &node_ids) const {
      FloatsList ret(node_ids.size());
      for (unsigned int i = 0; i< node_ids.size(); ++i) {
        NodeConstHandle nh = fh.get_node(node_ids[i]);
        ret[i].resize(coordinates_.size());
        for (unsigned int j = 0; j < coordinates_.size(); ++j) {
          ret[i][j] = nh.get_value(coordinates_[j]);
        }
      }
      return ret;
  }


  };

  typedef std::vector<IntermediateParticleFactory> IntermediateParticleFactories;


  /** These particles have associated coordinates and orientation information.

       See also RigidParticle and RigidParticleConstFactory.
    */
  class RigidParticleConst: public Decorator<NodeConstHandle> {
    friend class RigidParticleConstFactory;
    std::string get_name() const {return "RigidParticle";}
  private:
    typedef Decorator<NodeConstHandle> P;
FloatKeys orientation_;
FloatKeys coordinates_;
    RigidParticleConst(NodeConstHandle nh,
              FloatKeys orientation,
FloatKeys coordinates):
       P(nh),
orientation_(orientation),
coordinates_(coordinates) {
    }
  public:
  /** The orientation as a quaternion. */
  Floats get_orientation() const {
    try {
      Floats ret(orientation_.size());
      for (unsigned int i = 0; i < orientation_.size(); ++i) {
        ret[i] = P::get_value(orientation_[i]);
      }
      return ret;
    } RMF_DECORATOR_CATCH( );
  }

  /** The coordinates in angstroms. */
  Floats get_coordinates() const {
    try {
      Floats ret(coordinates_.size());
      for (unsigned int i = 0; i < coordinates_.size(); ++i) {
        ret[i] = P::get_value(coordinates_[i]);
      }
      return ret;
    } RMF_DECORATOR_CATCH( );
  }

    static std::string get_decorator_type_name() {
         return "RigidParticleConst";
    }
  };

    typedef std::vector<RigidParticleConst> RigidParticleConsts;

  /** These particles have associated coordinates and orientation information.

       See also RigidParticleConst and RigidParticleFactory.
    */
  class RigidParticle: public Decorator<NodeHandle> {
    friend class RigidParticleFactory;
    std::string get_name() const {return "RigidParticle";}
  private:
    typedef Decorator<NodeHandle> P;
FloatKeys orientation_;
FloatKeys coordinates_;
    RigidParticle(NodeHandle nh,
              FloatKeys orientation,
FloatKeys coordinates):
       P(nh),
orientation_(orientation),
coordinates_(coordinates) {
    }
  public:
  /** The orientation as a quaternion. */
  Floats get_orientation() const {
    try {
      Floats ret(orientation_.size());
      for (unsigned int i = 0; i < orientation_.size(); ++i) {
        ret[i] = P::get_value(orientation_[i]);
      }
      return ret;
    } RMF_DECORATOR_CATCH( );
  }
  /** The orientation as a quaternion. */
  void set_orientation(Floats v) {
    try {
      for (unsigned int i = 0; i< orientation_.size(); ++i) {
         P::set_value(orientation_[i], v[i]);
      }
    } RMF_DECORATOR_CATCH( );
  }

  /** The coordinates in angstroms. */
  Floats get_coordinates() const {
    try {
      Floats ret(coordinates_.size());
      for (unsigned int i = 0; i < coordinates_.size(); ++i) {
        ret[i] = P::get_value(coordinates_[i]);
      }
      return ret;
    } RMF_DECORATOR_CATCH( );
  }
  /** The coordinates in angstroms. */
  void set_coordinates(Floats v) {
    try {
      for (unsigned int i = 0; i< coordinates_.size(); ++i) {
         P::set_value(coordinates_[i], v[i]);
      }
    } RMF_DECORATOR_CATCH( );
  }

    static std::string get_decorator_type_name() {
         return "RigidParticle";
    }
  };

    typedef std::vector<RigidParticle> RigidParticles;

  /** Create decorators of type RigidParticle.

       See also RigidParticleConst and RigidParticleFactory.
    */
  class RigidParticleConstFactory: public Factory<FileConstHandle> {
    typedef Factory<FileConstHandle> P;
    Category cat_;
FloatKeys orientation_;
FloatKeys coordinates_;

  FloatKeys get_orientation_keys(FileConstHandle fh) {
    FloatKeys ret;
    ret.push_back(fh.get_key<FloatTraits>(cat_, "orientation r"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "orientation i"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "orientation j"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "orientation k"));;
    return ret;
  }
  bool get_has_orientation_attributes(NodeConstHandle nh) const {
    for (unsigned int i = 0; i< orientation_.size(); ++i) {
      if (!nh.get_has_value(orientation_[i])) return false;
    }
    return true;
  }


  FloatKeys get_coordinates_keys(FileConstHandle fh) {
    FloatKeys ret;
    ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian x"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian y"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian z"));;
    return ret;
  }
  bool get_has_coordinates_attributes(NodeConstHandle nh) const {
    for (unsigned int i = 0; i< coordinates_.size(); ++i) {
      if (!nh.get_has_value(coordinates_[i])) return false;
    }
    return true;
  }

  public:
    RigidParticleConstFactory(FileConstHandle fh):
    P(),
    cat_(fh.get_category("physics")),
    orientation_(get_orientation_keys(fh)), coordinates_(get_coordinates_keys(fh)) {
    }
    /** Get a RigidParticleConst for nh.*/
    RigidParticleConst get(NodeConstHandle nh) const {
      RMF_USAGE_CHECK((nh.get_type() == RMF::REPRESENTATION), std::string("Bad node type. Got \"")
                                      + boost::lexical_cast<std::string>(nh.get_type())
                                      + "\" in decorator type  RigidParticle");
      return RigidParticleConst(nh, orientation_,
coordinates_);
    }
    /** Check whether nh has all the attributes required to be a
        RigidParticleConst.*/
    bool get_is(NodeConstHandle nh) const {
      return (nh.get_type() == RMF::REPRESENTATION)
    && nh.get_has_value(orientation_[0]) && nh.get_has_value(orientation_[1]) && nh.get_has_value(orientation_[2]) && nh.get_has_value(orientation_[3])
    && nh.get_has_value(coordinates_[0]) && nh.get_has_value(coordinates_[1]) && nh.get_has_value(coordinates_[2]);
    }
    

  /** Get values from a list of NodeIDs all at the same time. This method
      is a bit experimental and may change.
    */
  FloatsList get_coordinates(FileConstHandle fh,
                       const NodeIDs &node_ids) const {
      FloatsList ret(node_ids.size());
      for (unsigned int i = 0; i< node_ids.size(); ++i) {
        NodeConstHandle nh = fh.get_node(node_ids[i]);
        ret[i].resize(coordinates_.size());
        for (unsigned int j = 0; j < coordinates_.size(); ++j) {
          ret[i][j] = nh.get_value(coordinates_[j]);
        }
      }
      return ret;
  }

  };

  typedef std::vector<RigidParticleConstFactory> RigidParticleConstFactories;

  /** Create decorators of type RigidParticle.

       See also RigidParticle and RigidParticleConstFactory.
    */
  class RigidParticleFactory: public Factory<FileHandle> {
    typedef Factory<FileHandle> P;
    Category cat_;
FloatKeys orientation_;
FloatKeys coordinates_;

  FloatKeys get_orientation_keys(FileHandle fh) {
    FloatKeys ret;
    ret.push_back(fh.get_key<FloatTraits>(cat_, "orientation r"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "orientation i"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "orientation j"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "orientation k"));;
    return ret;
  }
  bool get_has_orientation_attributes(NodeConstHandle nh) const {
    for (unsigned int i = 0; i< orientation_.size(); ++i) {
      if (!nh.get_has_value(orientation_[i])) return false;
    }
    return true;
  }


  FloatKeys get_coordinates_keys(FileHandle fh) {
    FloatKeys ret;
    ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian x"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian y"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian z"));;
    return ret;
  }
  bool get_has_coordinates_attributes(NodeConstHandle nh) const {
    for (unsigned int i = 0; i< coordinates_.size(); ++i) {
      if (!nh.get_has_value(coordinates_[i])) return false;
    }
    return true;
  }

  public:
    RigidParticleFactory(FileHandle fh):
    P(),
    cat_(fh.get_category("physics")),
    orientation_(get_orientation_keys(fh)), coordinates_(get_coordinates_keys(fh)) {
    }
    /** Get a RigidParticle for nh.*/
    RigidParticle get(NodeHandle nh) const {
      RMF_USAGE_CHECK((nh.get_type() == RMF::REPRESENTATION), std::string("Bad node type. Got \"")
                                      + boost::lexical_cast<std::string>(nh.get_type())
                                      + "\" in decorator type  RigidParticle");
                                      
                                      
      return RigidParticle(nh, orientation_,
coordinates_);
    }
    /** Check whether nh has all the attributes required to be a
        RigidParticle.*/
    bool get_is(NodeHandle nh) const {
      return (nh.get_type() == RMF::REPRESENTATION)
    && nh.get_has_value(orientation_[0]) && nh.get_has_value(orientation_[1]) && nh.get_has_value(orientation_[2]) && nh.get_has_value(orientation_[3])
    && nh.get_has_value(coordinates_[0]) && nh.get_has_value(coordinates_[1]) && nh.get_has_value(coordinates_[2]);
    }
    

  /** Get values from a list of NodeIDs all at the same time. This method
      is a bit experimental and may change.
    */
  FloatsList get_coordinates(FileConstHandle fh,
                       const NodeIDs &node_ids) const {
      FloatsList ret(node_ids.size());
      for (unsigned int i = 0; i< node_ids.size(); ++i) {
        NodeConstHandle nh = fh.get_node(node_ids[i]);
        ret[i].resize(coordinates_.size());
        for (unsigned int j = 0; j < coordinates_.size(); ++j) {
          ret[i][j] = nh.get_value(coordinates_[j]);
        }
      }
      return ret;
  }

  };

  typedef std::vector<RigidParticleFactory> RigidParticleFactories;


  /** Information regarding diffusion coefficients.

       See also Diffuser and DiffuserConstFactory.
    */
  class DiffuserConst: public Decorator<NodeConstHandle> {
    friend class DiffuserConstFactory;
    std::string get_name() const {return "Diffuser";}
  private:
    typedef Decorator<NodeConstHandle> P;
FloatKey diffusion_coefficient_;
    DiffuserConst(NodeConstHandle nh,
              FloatKey diffusion_coefficient):
       P(nh),
diffusion_coefficient_(diffusion_coefficient) {
    }
  public:
  /** The diffusion coefficient in A^2/fs. */
  Float get_diffusion_coefficient() const {
    try {
      return P::get_value(diffusion_coefficient_);
    } RMF_DECORATOR_CATCH( );
  }

    static std::string get_decorator_type_name() {
         return "DiffuserConst";
    }
  };

    typedef std::vector<DiffuserConst> DiffuserConsts;

  /** Information regarding diffusion coefficients.

       See also DiffuserConst and DiffuserFactory.
    */
  class Diffuser: public Decorator<NodeHandle> {
    friend class DiffuserFactory;
    std::string get_name() const {return "Diffuser";}
  private:
    typedef Decorator<NodeHandle> P;
FloatKey diffusion_coefficient_;
    Diffuser(NodeHandle nh,
              FloatKey diffusion_coefficient):
       P(nh),
diffusion_coefficient_(diffusion_coefficient) {
    }
  public:
  /** The diffusion coefficient in A^2/fs. */
  Float get_diffusion_coefficient() const {
    try {
      return P::get_value(diffusion_coefficient_);
    } RMF_DECORATOR_CATCH( );
  }
  /** The diffusion coefficient in A^2/fs. */
  void set_diffusion_coefficient(Float v) {
    try {
      P::set_value(diffusion_coefficient_, v);
    } RMF_DECORATOR_CATCH( );
  }


    static std::string get_decorator_type_name() {
         return "Diffuser";
    }
  };

    typedef std::vector<Diffuser> Diffusers;

  /** Create decorators of type Diffuser.

       See also DiffuserConst and DiffuserFactory.
    */
  class DiffuserConstFactory: public Factory<FileConstHandle> {
    typedef Factory<FileConstHandle> P;
    Category cat_;
FloatKey diffusion_coefficient_;

  public:
    DiffuserConstFactory(FileConstHandle fh):
    P(),
    cat_(fh.get_category("physics")),
    diffusion_coefficient_(fh.get_key<FloatTraits>(cat_, "diffusion coefficient")) {
    }
    /** Get a DiffuserConst for nh.*/
    DiffuserConst get(NodeConstHandle nh) const {
      RMF_USAGE_CHECK((nh.get_type() == RMF::REPRESENTATION), std::string("Bad node type. Got \"")
                                      + boost::lexical_cast<std::string>(nh.get_type())
                                      + "\" in decorator type  Diffuser");
      return DiffuserConst(nh, diffusion_coefficient_);
    }
    /** Check whether nh has all the attributes required to be a
        DiffuserConst.*/
    bool get_is(NodeConstHandle nh) const {
      return (nh.get_type() == RMF::REPRESENTATION)
    && nh.get_has_value(diffusion_coefficient_);
    }
    
  };

  typedef std::vector<DiffuserConstFactory> DiffuserConstFactories;

  /** Create decorators of type Diffuser.

       See also Diffuser and DiffuserConstFactory.
    */
  class DiffuserFactory: public Factory<FileHandle> {
    typedef Factory<FileHandle> P;
    Category cat_;
FloatKey diffusion_coefficient_;

  public:
    DiffuserFactory(FileHandle fh):
    P(),
    cat_(fh.get_category("physics")),
    diffusion_coefficient_(fh.get_key<FloatTraits>(cat_, "diffusion coefficient")) {
    }
    /** Get a Diffuser for nh.*/
    Diffuser get(NodeHandle nh) const {
      RMF_USAGE_CHECK((nh.get_type() == RMF::REPRESENTATION), std::string("Bad node type. Got \"")
                                      + boost::lexical_cast<std::string>(nh.get_type())
                                      + "\" in decorator type  Diffuser");
                                      
                                      
      return Diffuser(nh, diffusion_coefficient_);
    }
    /** Check whether nh has all the attributes required to be a
        Diffuser.*/
    bool get_is(NodeHandle nh) const {
      return (nh.get_type() == RMF::REPRESENTATION)
    && nh.get_has_value(diffusion_coefficient_);
    }
    
  };

  typedef std::vector<DiffuserFactory> DiffuserFactories;


  /** Information regarding an atom.

       See also Atom and AtomConstFactory.
    */
  class AtomConst: public Decorator<NodeConstHandle> {
    friend class AtomConstFactory;
    std::string get_name() const {return "Atom";}
  private:
    typedef Decorator<NodeConstHandle> P;
FloatKeys coordinates_;
FloatKey mass_;
FloatKey radius_;
IndexKey element_;
    AtomConst(NodeConstHandle nh,
              FloatKeys coordinates,
FloatKey mass,
FloatKey radius,
IndexKey element):
       P(nh),
coordinates_(coordinates),
mass_(mass),
radius_(radius),
element_(element) {
    }
  public:
  /** The coordinates in angstroms. */
  Floats get_coordinates() const {
    try {
      Floats ret(coordinates_.size());
      for (unsigned int i = 0; i < coordinates_.size(); ++i) {
        ret[i] = P::get_value(coordinates_[i]);
      }
      return ret;
    } RMF_DECORATOR_CATCH( );
  }

  /** The mass in Daltons. */
  Float get_mass() const {
    try {
      return P::get_value(mass_);
    } RMF_DECORATOR_CATCH( );
  }

  /** The radius in angstroms. */
  Float get_radius() const {
    try {
      return P::get_value(radius_);
    } RMF_DECORATOR_CATCH( );
  }

  /** The atomic number of the element. */
  Index get_element() const {
    try {
      return P::get_value(element_);
    } RMF_DECORATOR_CATCH( );
  }

    static std::string get_decorator_type_name() {
         return "AtomConst";
    }
  };

    typedef std::vector<AtomConst> AtomConsts;

  /** Information regarding an atom.

       See also AtomConst and AtomFactory.
    */
  class Atom: public Decorator<NodeHandle> {
    friend class AtomFactory;
    std::string get_name() const {return "Atom";}
  private:
    typedef Decorator<NodeHandle> P;
FloatKeys coordinates_;
FloatKey mass_;
FloatKey radius_;
IndexKey element_;
    Atom(NodeHandle nh,
              FloatKeys coordinates,
FloatKey mass,
FloatKey radius,
IndexKey element):
       P(nh),
coordinates_(coordinates),
mass_(mass),
radius_(radius),
element_(element) {
    }
  public:
  /** The coordinates in angstroms. */
  Floats get_coordinates() const {
    try {
      Floats ret(coordinates_.size());
      for (unsigned int i = 0; i < coordinates_.size(); ++i) {
        ret[i] = P::get_value(coordinates_[i]);
      }
      return ret;
    } RMF_DECORATOR_CATCH( );
  }
  /** The coordinates in angstroms. */
  void set_coordinates(Floats v) {
    try {
      for (unsigned int i = 0; i< coordinates_.size(); ++i) {
         P::set_value(coordinates_[i], v[i]);
      }
    } RMF_DECORATOR_CATCH( );
  }

  /** The mass in Daltons. */
  Float get_mass() const {
    try {
      return P::get_value(mass_);
    } RMF_DECORATOR_CATCH( );
  }
  /** The mass in Daltons. */
  void set_mass(Float v) {
    try {
      P::set_value(mass_, v);
    } RMF_DECORATOR_CATCH( );
  }


  /** The radius in angstroms. */
  Float get_radius() const {
    try {
      return P::get_value(radius_);
    } RMF_DECORATOR_CATCH( );
  }
  /** The radius in angstroms. */
  void set_radius(Float v) {
    try {
      P::set_value(radius_, v);
    } RMF_DECORATOR_CATCH( );
  }


  /** The atomic number of the element. */
  Index get_element() const {
    try {
      return P::get_value(element_);
    } RMF_DECORATOR_CATCH( );
  }
  /** The atomic number of the element. */
  void set_element(Index v) {
    try {
      P::set_value(element_, v);
    } RMF_DECORATOR_CATCH( );
  }


    static std::string get_decorator_type_name() {
         return "Atom";
    }
  };

    typedef std::vector<Atom> Atoms;

  /** Create decorators of type Atom.

       See also AtomConst and AtomFactory.
    */
  class AtomConstFactory: public Factory<FileConstHandle> {
    typedef Factory<FileConstHandle> P;
    Category cat_;
FloatKeys coordinates_;
FloatKey mass_;
FloatKey radius_;
IndexKey element_;

  FloatKeys get_coordinates_keys(FileConstHandle fh) {
    FloatKeys ret;
    ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian x"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian y"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian z"));;
    return ret;
  }
  bool get_has_coordinates_attributes(NodeConstHandle nh) const {
    for (unsigned int i = 0; i< coordinates_.size(); ++i) {
      if (!nh.get_has_value(coordinates_[i])) return false;
    }
    return true;
  }




  public:
    AtomConstFactory(FileConstHandle fh):
    P(),
    cat_(fh.get_category("physics")),
    coordinates_(get_coordinates_keys(fh)), mass_(fh.get_key<FloatTraits>(cat_, "mass")), radius_(fh.get_key<FloatTraits>(cat_, "radius")), element_(fh.get_key<IndexTraits>(cat_, "element")) {
    }
    /** Get a AtomConst for nh.*/
    AtomConst get(NodeConstHandle nh) const {
      RMF_USAGE_CHECK((nh.get_type() == RMF::REPRESENTATION), std::string("Bad node type. Got \"")
                                      + boost::lexical_cast<std::string>(nh.get_type())
                                      + "\" in decorator type  Atom");
      return AtomConst(nh, coordinates_,
mass_,
radius_,
element_);
    }
    /** Check whether nh has all the attributes required to be a
        AtomConst.*/
    bool get_is(NodeConstHandle nh) const {
      return (nh.get_type() == RMF::REPRESENTATION)
    && nh.get_has_value(coordinates_[0]) && nh.get_has_value(coordinates_[1]) && nh.get_has_value(coordinates_[2])
    && nh.get_has_value(mass_)
    && nh.get_has_value(radius_)
    && nh.get_has_value(element_);
    }
    
  /** Get values from a list of NodeIDs all at the same time. This method
      is a bit experimental and may change.
    */
  FloatsList get_coordinates(FileConstHandle fh,
                       const NodeIDs &node_ids) const {
      FloatsList ret(node_ids.size());
      for (unsigned int i = 0; i< node_ids.size(); ++i) {
        NodeConstHandle nh = fh.get_node(node_ids[i]);
        ret[i].resize(coordinates_.size());
        for (unsigned int j = 0; j < coordinates_.size(); ++j) {
          ret[i][j] = nh.get_value(coordinates_[j]);
        }
      }
      return ret;
  }




  };

  typedef std::vector<AtomConstFactory> AtomConstFactories;

  /** Create decorators of type Atom.

       See also Atom and AtomConstFactory.
    */
  class AtomFactory: public Factory<FileHandle> {
    typedef Factory<FileHandle> P;
    Category cat_;
FloatKeys coordinates_;
FloatKey mass_;
FloatKey radius_;
IndexKey element_;

  FloatKeys get_coordinates_keys(FileHandle fh) {
    FloatKeys ret;
    ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian x"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian y"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "cartesian z"));;
    return ret;
  }
  bool get_has_coordinates_attributes(NodeConstHandle nh) const {
    for (unsigned int i = 0; i< coordinates_.size(); ++i) {
      if (!nh.get_has_value(coordinates_[i])) return false;
    }
    return true;
  }




  public:
    AtomFactory(FileHandle fh):
    P(),
    cat_(fh.get_category("physics")),
    coordinates_(get_coordinates_keys(fh)), mass_(fh.get_key<FloatTraits>(cat_, "mass")), radius_(fh.get_key<FloatTraits>(cat_, "radius")), element_(fh.get_key<IndexTraits>(cat_, "element")) {
    }
    /** Get a Atom for nh.*/
    Atom get(NodeHandle nh) const {
      RMF_USAGE_CHECK((nh.get_type() == RMF::REPRESENTATION), std::string("Bad node type. Got \"")
                                      + boost::lexical_cast<std::string>(nh.get_type())
                                      + "\" in decorator type  Atom");
                                      
                                      
      return Atom(nh, coordinates_,
mass_,
radius_,
element_);
    }
    /** Check whether nh has all the attributes required to be a
        Atom.*/
    bool get_is(NodeHandle nh) const {
      return (nh.get_type() == RMF::REPRESENTATION)
    && nh.get_has_value(coordinates_[0]) && nh.get_has_value(coordinates_[1]) && nh.get_has_value(coordinates_[2])
    && nh.get_has_value(mass_)
    && nh.get_has_value(radius_)
    && nh.get_has_value(element_);
    }
    
  /** Get values from a list of NodeIDs all at the same time. This method
      is a bit experimental and may change.
    */
  FloatsList get_coordinates(FileConstHandle fh,
                       const NodeIDs &node_ids) const {
      FloatsList ret(node_ids.size());
      for (unsigned int i = 0; i< node_ids.size(); ++i) {
        NodeConstHandle nh = fh.get_node(node_ids[i]);
        ret[i].resize(coordinates_.size());
        for (unsigned int j = 0; j < coordinates_.size(); ++j) {
          ret[i][j] = nh.get_value(coordinates_[j]);
        }
      }
      return ret;
  }




  };

  typedef std::vector<AtomFactory> AtomFactories;


  /** A bond between particles.

       See also Bond and BondConstFactory.
    */
  class BondConst: public Decorator<NodeConstHandle> {
    friend class BondConstFactory;
    std::string get_name() const {return "Bond";}
  private:
    typedef Decorator<NodeConstHandle> P;
AliasConstFactory bonded_;
    BondConst(NodeConstHandle nh,
              AliasConstFactory bonded):
       P(nh),
bonded_(bonded) {
    }
  public:
  /** The bonded particles. */
  NodeConstHandles get_bonded() const {
    try {
       NodeConstHandles typed = get_node().get_children();
       NodeConstHandles ret;
       for (unsigned int i = 0; i < typed.size(); ++i) {
         if (bonded_.get_is(typed[i])) {
           ret.push_back(bonded_.get(typed[i]).get_aliased());
         }
       }
       return ret;
    } RMF_DECORATOR_CATCH( );
  }

    static std::string get_decorator_type_name() {
         return "BondConst";
    }
  };

    typedef std::vector<BondConst> BondConsts;

  /** A bond between particles.

       See also BondConst and BondFactory.
    */
  class Bond: public Decorator<NodeHandle> {
    friend class BondFactory;
    std::string get_name() const {return "Bond";}
  private:
    typedef Decorator<NodeHandle> P;
AliasFactory bonded_;
    Bond(NodeHandle nh,
              AliasFactory bonded):
       P(nh),
bonded_(bonded) {
    }
  public:
  /** The bonded particles. */
  NodeHandles get_bonded() const {
    try {
       NodeHandles typed = get_node().get_children();
       NodeHandles ret;
       for (unsigned int i = 0; i < typed.size(); ++i) {
         if (bonded_.get_is(typed[i])) {
           ret.push_back(bonded_.get(typed[i]).get_aliased());
         }
       }
       return ret;
    } RMF_DECORATOR_CATCH( );
  }
  /** The bonded particles. */
  void set_bonded(const NodeConstHandles &v) {
    try {
       for (unsigned int i = 0; i < v.size(); ++i) {
         internal::add_child_alias(bonded_, get_node(), v[i]);
       }
    } RMF_DECORATOR_CATCH( );
  }
  void set_bonded(const NodeHandles &v) {
    try {
       for (unsigned int i = 0; i < v.size(); ++i) {
         internal::add_child_alias(bonded_, get_node(), v[i]);
       }
    } RMF_DECORATOR_CATCH( );
  }

    static std::string get_decorator_type_name() {
         return "Bond";
    }
  };

    typedef std::vector<Bond> Bonds;

  /** Create decorators of type Bond.

       See also BondConst and BondFactory.
    */
  class BondConstFactory: public Factory<FileConstHandle> {
    typedef Factory<FileConstHandle> P;
    Category cat_;
AliasConstFactory bonded_;

  public:
    BondConstFactory(FileConstHandle fh):
    P(),
    cat_(fh.get_category("physics")),
    bonded_(fh) {
    }
    /** Get a BondConst for nh.*/
    BondConst get(NodeConstHandle nh) const {
      RMF_USAGE_CHECK((nh.get_type() == RMF::BOND), std::string("Bad node type. Got \"")
                                      + boost::lexical_cast<std::string>(nh.get_type())
                                      + "\" in decorator type  Bond");
      return BondConst(nh, bonded_);
    }
    /** Check whether nh has all the attributes required to be a
        BondConst.*/
    bool get_is(NodeConstHandle nh) const {
      return (nh.get_type() == RMF::BOND);
    }
    
  };

  typedef std::vector<BondConstFactory> BondConstFactories;

  /** Create decorators of type Bond.

       See also Bond and BondConstFactory.
    */
  class BondFactory: public Factory<FileHandle> {
    typedef Factory<FileHandle> P;
    Category cat_;
AliasFactory bonded_;

  public:
    BondFactory(FileHandle fh):
    P(),
    cat_(fh.get_category("physics")),
    bonded_(fh) {
    }
    /** Get a Bond for nh.*/
    Bond get(NodeHandle nh) const {
      RMF_USAGE_CHECK((nh.get_type() == RMF::BOND), std::string("Bad node type. Got \"")
                                      + boost::lexical_cast<std::string>(nh.get_type())
                                      + "\" in decorator type  Bond");
                                      
                                      
      return Bond(nh, bonded_);
    }
    /** Check whether nh has all the attributes required to be a
        Bond.*/
    bool get_is(NodeHandle nh) const {
      return (nh.get_type() == RMF::BOND);
    }
    
  };

  typedef std::vector<BondFactory> BondFactories;


  /** Define a transformation to be applied the the attributes of this and child particles, relative to the reference frame of the parent.

       See also ReferenceFrame and ReferenceFrameConstFactory.
    */
  class ReferenceFrameConst: public Decorator<NodeConstHandle> {
    friend class ReferenceFrameConstFactory;
    std::string get_name() const {return "ReferenceFrame";}
  private:
    typedef Decorator<NodeConstHandle> P;
FloatKeys rotation_;
FloatKeys translation_;
    ReferenceFrameConst(NodeConstHandle nh,
              FloatKeys rotation,
FloatKeys translation):
       P(nh),
rotation_(rotation),
translation_(translation) {
    }
  public:
  /** The rotational part of the relative transformation as a quaternion. */
  Floats get_rotation() const {
    try {
      Floats ret(rotation_.size());
      for (unsigned int i = 0; i < rotation_.size(); ++i) {
        ret[i] = P::get_value(rotation_[i]);
      }
      return ret;
    } RMF_DECORATOR_CATCH( );
  }

  /** The translation part of the relative transformion in angstroms. */
  Floats get_translation() const {
    try {
      Floats ret(translation_.size());
      for (unsigned int i = 0; i < translation_.size(); ++i) {
        ret[i] = P::get_value(translation_[i]);
      }
      return ret;
    } RMF_DECORATOR_CATCH( );
  }

    static std::string get_decorator_type_name() {
         return "ReferenceFrameConst";
    }
  };

    typedef std::vector<ReferenceFrameConst> ReferenceFrameConsts;

  /** Define a transformation to be applied the the attributes of this and child particles, relative to the reference frame of the parent.

       See also ReferenceFrameConst and ReferenceFrameFactory.
    */
  class ReferenceFrame: public Decorator<NodeHandle> {
    friend class ReferenceFrameFactory;
    std::string get_name() const {return "ReferenceFrame";}
  private:
    typedef Decorator<NodeHandle> P;
FloatKeys rotation_;
FloatKeys translation_;
    ReferenceFrame(NodeHandle nh,
              FloatKeys rotation,
FloatKeys translation):
       P(nh),
rotation_(rotation),
translation_(translation) {
    }
  public:
  /** The rotational part of the relative transformation as a quaternion. */
  Floats get_rotation() const {
    try {
      Floats ret(rotation_.size());
      for (unsigned int i = 0; i < rotation_.size(); ++i) {
        ret[i] = P::get_value(rotation_[i]);
      }
      return ret;
    } RMF_DECORATOR_CATCH( );
  }
  /** The rotational part of the relative transformation as a quaternion. */
  void set_rotation(Floats v) {
    try {
      for (unsigned int i = 0; i< rotation_.size(); ++i) {
         P::set_value(rotation_[i], v[i]);
      }
    } RMF_DECORATOR_CATCH( );
  }

  /** The translation part of the relative transformion in angstroms. */
  Floats get_translation() const {
    try {
      Floats ret(translation_.size());
      for (unsigned int i = 0; i < translation_.size(); ++i) {
        ret[i] = P::get_value(translation_[i]);
      }
      return ret;
    } RMF_DECORATOR_CATCH( );
  }
  /** The translation part of the relative transformion in angstroms. */
  void set_translation(Floats v) {
    try {
      for (unsigned int i = 0; i< translation_.size(); ++i) {
         P::set_value(translation_[i], v[i]);
      }
    } RMF_DECORATOR_CATCH( );
  }

    static std::string get_decorator_type_name() {
         return "ReferenceFrame";
    }
  };

    typedef std::vector<ReferenceFrame> ReferenceFrames;

  /** Create decorators of type ReferenceFrame.

       See also ReferenceFrameConst and ReferenceFrameFactory.
    */
  class ReferenceFrameConstFactory: public Factory<FileConstHandle> {
    typedef Factory<FileConstHandle> P;
    Category cat_;
FloatKeys rotation_;
FloatKeys translation_;

  FloatKeys get_rotation_keys(FileConstHandle fh) {
    FloatKeys ret;
    ret.push_back(fh.get_key<FloatTraits>(cat_, "reference frame orientation r"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "reference frame orientation i"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "reference frame orientation j"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "reference frame orientation k"));;
    return ret;
  }
  bool get_has_rotation_attributes(NodeConstHandle nh) const {
    for (unsigned int i = 0; i< rotation_.size(); ++i) {
      if (!nh.get_has_value(rotation_[i])) return false;
    }
    return true;
  }


  FloatKeys get_translation_keys(FileConstHandle fh) {
    FloatKeys ret;
    ret.push_back(fh.get_key<FloatTraits>(cat_, "reference frame cartesian x"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "reference frame cartesian y"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "reference frame cartesian z"));;
    return ret;
  }
  bool get_has_translation_attributes(NodeConstHandle nh) const {
    for (unsigned int i = 0; i< translation_.size(); ++i) {
      if (!nh.get_has_value(translation_[i])) return false;
    }
    return true;
  }

  public:
    ReferenceFrameConstFactory(FileConstHandle fh):
    P(),
    cat_(fh.get_category("physics")),
    rotation_(get_rotation_keys(fh)), translation_(get_translation_keys(fh)) {
    }
    /** Get a ReferenceFrameConst for nh.*/
    ReferenceFrameConst get(NodeConstHandle nh) const {
      RMF_USAGE_CHECK((nh.get_type() == RMF::REPRESENTATION||nh.get_type() == RMF::ORGANIZATIONAL), std::string("Bad node type. Got \"")
                                      + boost::lexical_cast<std::string>(nh.get_type())
                                      + "\" in decorator type  ReferenceFrame");
      return ReferenceFrameConst(nh, rotation_,
translation_);
    }
    /** Check whether nh has all the attributes required to be a
        ReferenceFrameConst.*/
    bool get_is(NodeConstHandle nh) const {
      return (nh.get_type() == RMF::REPRESENTATION||nh.get_type() == RMF::ORGANIZATIONAL)
    && nh.get_has_value(rotation_[0]) && nh.get_has_value(rotation_[1]) && nh.get_has_value(rotation_[2]) && nh.get_has_value(rotation_[3])
    && nh.get_has_value(translation_[0]) && nh.get_has_value(translation_[1]) && nh.get_has_value(translation_[2]);
    }
    

  };

  typedef std::vector<ReferenceFrameConstFactory> ReferenceFrameConstFactories;

  /** Create decorators of type ReferenceFrame.

       See also ReferenceFrame and ReferenceFrameConstFactory.
    */
  class ReferenceFrameFactory: public Factory<FileHandle> {
    typedef Factory<FileHandle> P;
    Category cat_;
FloatKeys rotation_;
FloatKeys translation_;

  FloatKeys get_rotation_keys(FileHandle fh) {
    FloatKeys ret;
    ret.push_back(fh.get_key<FloatTraits>(cat_, "reference frame orientation r"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "reference frame orientation i"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "reference frame orientation j"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "reference frame orientation k"));;
    return ret;
  }
  bool get_has_rotation_attributes(NodeConstHandle nh) const {
    for (unsigned int i = 0; i< rotation_.size(); ++i) {
      if (!nh.get_has_value(rotation_[i])) return false;
    }
    return true;
  }


  FloatKeys get_translation_keys(FileHandle fh) {
    FloatKeys ret;
    ret.push_back(fh.get_key<FloatTraits>(cat_, "reference frame cartesian x"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "reference frame cartesian y"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "reference frame cartesian z"));;
    return ret;
  }
  bool get_has_translation_attributes(NodeConstHandle nh) const {
    for (unsigned int i = 0; i< translation_.size(); ++i) {
      if (!nh.get_has_value(translation_[i])) return false;
    }
    return true;
  }

  public:
    ReferenceFrameFactory(FileHandle fh):
    P(),
    cat_(fh.get_category("physics")),
    rotation_(get_rotation_keys(fh)), translation_(get_translation_keys(fh)) {
    }
    /** Get a ReferenceFrame for nh.*/
    ReferenceFrame get(NodeHandle nh) const {
      RMF_USAGE_CHECK((nh.get_type() == RMF::REPRESENTATION||nh.get_type() == RMF::ORGANIZATIONAL), std::string("Bad node type. Got \"")
                                      + boost::lexical_cast<std::string>(nh.get_type())
                                      + "\" in decorator type  ReferenceFrame");
                                      
                                      
      return ReferenceFrame(nh, rotation_,
translation_);
    }
    /** Check whether nh has all the attributes required to be a
        ReferenceFrame.*/
    bool get_is(NodeHandle nh) const {
      return (nh.get_type() == RMF::REPRESENTATION||nh.get_type() == RMF::ORGANIZATIONAL)
    && nh.get_has_value(rotation_[0]) && nh.get_has_value(rotation_[1]) && nh.get_has_value(rotation_[2]) && nh.get_has_value(rotation_[3])
    && nh.get_has_value(translation_[0]) && nh.get_has_value(translation_[1]) && nh.get_has_value(translation_[2]);
    }
    

  };

  typedef std::vector<ReferenceFrameFactory> ReferenceFrameFactories;


  /** Forces acting on particles in kCal/mol/A.

       See also Force and ForceConstFactory.
    */
  class ForceConst: public Decorator<NodeConstHandle> {
    friend class ForceConstFactory;
    std::string get_name() const {return "Force";}
  private:
    typedef Decorator<NodeConstHandle> P;
FloatKeys force_;
    ForceConst(NodeConstHandle nh,
              FloatKeys force):
       P(nh),
force_(force) {
    }
  public:
  /** The force. */
  Floats get_force() const {
    try {
      Floats ret(force_.size());
      for (unsigned int i = 0; i < force_.size(); ++i) {
        ret[i] = P::get_value(force_[i]);
      }
      return ret;
    } RMF_DECORATOR_CATCH( );
  }

    static std::string get_decorator_type_name() {
         return "ForceConst";
    }
  };

    typedef std::vector<ForceConst> ForceConsts;

  /** Forces acting on particles in kCal/mol/A.

       See also ForceConst and ForceFactory.
    */
  class Force: public Decorator<NodeHandle> {
    friend class ForceFactory;
    std::string get_name() const {return "Force";}
  private:
    typedef Decorator<NodeHandle> P;
FloatKeys force_;
    Force(NodeHandle nh,
              FloatKeys force):
       P(nh),
force_(force) {
    }
  public:
  /** The force. */
  Floats get_force() const {
    try {
      Floats ret(force_.size());
      for (unsigned int i = 0; i < force_.size(); ++i) {
        ret[i] = P::get_value(force_[i]);
      }
      return ret;
    } RMF_DECORATOR_CATCH( );
  }
  /** The force. */
  void set_force(Floats v) {
    try {
      for (unsigned int i = 0; i< force_.size(); ++i) {
         P::set_value(force_[i], v[i]);
      }
    } RMF_DECORATOR_CATCH( );
  }

    static std::string get_decorator_type_name() {
         return "Force";
    }
  };

    typedef std::vector<Force> Forces;

  /** Create decorators of type Force.

       See also ForceConst and ForceFactory.
    */
  class ForceConstFactory: public Factory<FileConstHandle> {
    typedef Factory<FileConstHandle> P;
    Category cat_;
FloatKeys force_;

  FloatKeys get_force_keys(FileConstHandle fh) {
    FloatKeys ret;
    ret.push_back(fh.get_key<FloatTraits>(cat_, "force cartesian x"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "force cartesian y"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "force cartesian z"));;
    return ret;
  }
  bool get_has_force_attributes(NodeConstHandle nh) const {
    for (unsigned int i = 0; i< force_.size(); ++i) {
      if (!nh.get_has_value(force_[i])) return false;
    }
    return true;
  }

  public:
    ForceConstFactory(FileConstHandle fh):
    P(),
    cat_(fh.get_category("physics")),
    force_(get_force_keys(fh)) {
    }
    /** Get a ForceConst for nh.*/
    ForceConst get(NodeConstHandle nh) const {
      RMF_USAGE_CHECK((nh.get_type() == RMF::REPRESENTATION), std::string("Bad node type. Got \"")
                                      + boost::lexical_cast<std::string>(nh.get_type())
                                      + "\" in decorator type  Force");
      return ForceConst(nh, force_);
    }
    /** Check whether nh has all the attributes required to be a
        ForceConst.*/
    bool get_is(NodeConstHandle nh) const {
      return (nh.get_type() == RMF::REPRESENTATION)
    && nh.get_has_value(force_[0]) && nh.get_has_value(force_[1]) && nh.get_has_value(force_[2]);
    }
    
  };

  typedef std::vector<ForceConstFactory> ForceConstFactories;

  /** Create decorators of type Force.

       See also Force and ForceConstFactory.
    */
  class ForceFactory: public Factory<FileHandle> {
    typedef Factory<FileHandle> P;
    Category cat_;
FloatKeys force_;

  FloatKeys get_force_keys(FileHandle fh) {
    FloatKeys ret;
    ret.push_back(fh.get_key<FloatTraits>(cat_, "force cartesian x"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "force cartesian y"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "force cartesian z"));;
    return ret;
  }
  bool get_has_force_attributes(NodeConstHandle nh) const {
    for (unsigned int i = 0; i< force_.size(); ++i) {
      if (!nh.get_has_value(force_[i])) return false;
    }
    return true;
  }

  public:
    ForceFactory(FileHandle fh):
    P(),
    cat_(fh.get_category("physics")),
    force_(get_force_keys(fh)) {
    }
    /** Get a Force for nh.*/
    Force get(NodeHandle nh) const {
      RMF_USAGE_CHECK((nh.get_type() == RMF::REPRESENTATION), std::string("Bad node type. Got \"")
                                      + boost::lexical_cast<std::string>(nh.get_type())
                                      + "\" in decorator type  Force");
                                      
                                      
      return Force(nh, force_);
    }
    /** Check whether nh has all the attributes required to be a
        Force.*/
    bool get_is(NodeHandle nh) const {
      return (nh.get_type() == RMF::REPRESENTATION)
    && nh.get_has_value(force_[0]) && nh.get_has_value(force_[1]) && nh.get_has_value(force_[2]);
    }
    
  };

  typedef std::vector<ForceFactory> ForceFactories;


  /** Torque acting on particles in kCal/mol/radian.

       See also Torque and TorqueConstFactory.
    */
  class TorqueConst: public Decorator<NodeConstHandle> {
    friend class TorqueConstFactory;
    std::string get_name() const {return "Torque";}
  private:
    typedef Decorator<NodeConstHandle> P;
FloatKeys torque_;
    TorqueConst(NodeConstHandle nh,
              FloatKeys torque):
       P(nh),
torque_(torque) {
    }
  public:
  /** The torque. */
  Floats get_torque() const {
    try {
      Floats ret(torque_.size());
      for (unsigned int i = 0; i < torque_.size(); ++i) {
        ret[i] = P::get_value(torque_[i]);
      }
      return ret;
    } RMF_DECORATOR_CATCH( );
  }

    static std::string get_decorator_type_name() {
         return "TorqueConst";
    }
  };

    typedef std::vector<TorqueConst> TorqueConsts;

  /** Torque acting on particles in kCal/mol/radian.

       See also TorqueConst and TorqueFactory.
    */
  class Torque: public Decorator<NodeHandle> {
    friend class TorqueFactory;
    std::string get_name() const {return "Torque";}
  private:
    typedef Decorator<NodeHandle> P;
FloatKeys torque_;
    Torque(NodeHandle nh,
              FloatKeys torque):
       P(nh),
torque_(torque) {
    }
  public:
  /** The torque. */
  Floats get_torque() const {
    try {
      Floats ret(torque_.size());
      for (unsigned int i = 0; i < torque_.size(); ++i) {
        ret[i] = P::get_value(torque_[i]);
      }
      return ret;
    } RMF_DECORATOR_CATCH( );
  }
  /** The torque. */
  void set_torque(Floats v) {
    try {
      for (unsigned int i = 0; i< torque_.size(); ++i) {
         P::set_value(torque_[i], v[i]);
      }
    } RMF_DECORATOR_CATCH( );
  }

    static std::string get_decorator_type_name() {
         return "Torque";
    }
  };

    typedef std::vector<Torque> Torques;

  /** Create decorators of type Torque.

       See also TorqueConst and TorqueFactory.
    */
  class TorqueConstFactory: public Factory<FileConstHandle> {
    typedef Factory<FileConstHandle> P;
    Category cat_;
FloatKeys torque_;

  FloatKeys get_torque_keys(FileConstHandle fh) {
    FloatKeys ret;
    ret.push_back(fh.get_key<FloatTraits>(cat_, "torque cartesian x"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "torque cartesian y"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "torque cartesian z"));;
    return ret;
  }
  bool get_has_torque_attributes(NodeConstHandle nh) const {
    for (unsigned int i = 0; i< torque_.size(); ++i) {
      if (!nh.get_has_value(torque_[i])) return false;
    }
    return true;
  }

  public:
    TorqueConstFactory(FileConstHandle fh):
    P(),
    cat_(fh.get_category("physics")),
    torque_(get_torque_keys(fh)) {
    }
    /** Get a TorqueConst for nh.*/
    TorqueConst get(NodeConstHandle nh) const {
      RMF_USAGE_CHECK((nh.get_type() == RMF::REPRESENTATION), std::string("Bad node type. Got \"")
                                      + boost::lexical_cast<std::string>(nh.get_type())
                                      + "\" in decorator type  Torque");
      return TorqueConst(nh, torque_);
    }
    /** Check whether nh has all the attributes required to be a
        TorqueConst.*/
    bool get_is(NodeConstHandle nh) const {
      return (nh.get_type() == RMF::REPRESENTATION)
    && nh.get_has_value(torque_[0]) && nh.get_has_value(torque_[1]) && nh.get_has_value(torque_[2]);
    }
    
  };

  typedef std::vector<TorqueConstFactory> TorqueConstFactories;

  /** Create decorators of type Torque.

       See also Torque and TorqueConstFactory.
    */
  class TorqueFactory: public Factory<FileHandle> {
    typedef Factory<FileHandle> P;
    Category cat_;
FloatKeys torque_;

  FloatKeys get_torque_keys(FileHandle fh) {
    FloatKeys ret;
    ret.push_back(fh.get_key<FloatTraits>(cat_, "torque cartesian x"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "torque cartesian y"));;ret.push_back(fh.get_key<FloatTraits>(cat_, "torque cartesian z"));;
    return ret;
  }
  bool get_has_torque_attributes(NodeConstHandle nh) const {
    for (unsigned int i = 0; i< torque_.size(); ++i) {
      if (!nh.get_has_value(torque_[i])) return false;
    }
    return true;
  }

  public:
    TorqueFactory(FileHandle fh):
    P(),
    cat_(fh.get_category("physics")),
    torque_(get_torque_keys(fh)) {
    }
    /** Get a Torque for nh.*/
    Torque get(NodeHandle nh) const {
      RMF_USAGE_CHECK((nh.get_type() == RMF::REPRESENTATION), std::string("Bad node type. Got \"")
                                      + boost::lexical_cast<std::string>(nh.get_type())
                                      + "\" in decorator type  Torque");
                                      
                                      
      return Torque(nh, torque_);
    }
    /** Check whether nh has all the attributes required to be a
        Torque.*/
    bool get_is(NodeHandle nh) const {
      return (nh.get_type() == RMF::REPRESENTATION)
    && nh.get_has_value(torque_[0]) && nh.get_has_value(torque_[1]) && nh.get_has_value(torque_[2]);
    }
    
  };

  typedef std::vector<TorqueFactory> TorqueFactories;

} /* namespace RMF */
RMF_DISABLE_WARNINGS

#endif /* RMF_PHYSICS_DECORATORS_H */
